#
# Author: Seth Vargo <sethvargo@gmail.com>
#
# Copyright 2014 Chef Software, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

module Falcore
  class Config < Hash
    class << self
      def from_file(path)
        contents = File.read(File.expand_path(path))
        parse(contents)
      rescue Errno::ENOENT
        raise "No config found at '#{path}'!"
      end

      def parse(contents)
        Parser.new(contents).parse
      end

      private

      class Parser
        CONFIG_SECTION = /\[(.+)\]/.freeze
        CONFIG_OPTION  = /[[:space:]]{2,}(.+) = (.+)/.freeze

        def initialize(contents)
          @contents = contents
          @result   = Config.new
        end

        def parse
          @contents.split(/\r?\n/).each do |line|
            next if line.strip.empty?
            next if line.strip.start_with?('#')

            if line =~ CONFIG_SECTION
              @current_key = $1
            elsif line =~ CONFIG_OPTION
              match = line.match(CONFIG_OPTION)
              key   = match[1]
              value = match[2]

              # Make sure we are keyed
              @result[@current_key] ||= Config.new
              @result[@current_key][key] = coerce(value)
            else
              raise "Could not parse line '#{line}'"
            end
          end

          @result
        end

        private

        #
        # Coerce the value into it's appropiate Ruby type.
        #
        # @param [Object] value
        # @return [Object]
        #   the coerced value
        #
        def coerce(value)
          case value
          when /^[[:digit:]]+$/
            value.to_i
          when /^[[:digit:]]+\.[[:digit:]]+$/
            value.to_f
          else
            value
          end
        end
      end
    end

    # @private
    def method_missing(m, *args, &block)
      key = m.to_s

      if key.include?('=')
        set(key.delete('='), args.first)
      else
        if has_key?(key)
          get(key)
        else
          NullObject.new
        end
      end
    end

    # @private
    def respond_to_missing?(m, include_private = false)
      key = m.to_s

      if key.include?('=')
        true
      else
        has_key?(key) || super
      end
    end

    private

    def get(key)
      self[key]
    end

    def set(key, value)
      self[key] = value
    end
  end
end
